@@ -1,7 +1,7 @@
 /**************************************************************************
  *   files.c  --  This file is part of GNU nano.                          *
  *                                                                        *
- *   Copyright (C) 1999-2011, 2013-2020 Free Software Foundation, Inc.    *
+ *   Copyright (C) 1999-2011, 2013-2021 Free Software Foundation, Inc.    *
  *   Copyright (C) 2015-2020 Benno Schulenberg                            *
  *                                                                        *
  *   GNU nano is free software: you can redistribute it and/or modify     *
@@ -81,6 +81,7 @@ void make_new_buffer(void)
 #endif
 #ifndef NANO_TINY
 	openfile->mark = NULL;
+	openfile->softmark = FALSE;
 
 	openfile->fmt = NIX_FILE;
 
@@ -97,6 +98,26 @@ void make_new_buffer(void)
 #endif
 }
 
+/* Return the given file name in a way that fits within the given space. */
+char *crop_to_fit(const char *name, int room)
+{
+	char *clipped;
+
+	if (breadth(name) <= room)
+		return display_string(name, 0, room, FALSE, FALSE);
+
+	if (room < 4)
+		return copy_of("_");
+
+	clipped = display_string(name, breadth(name) - room + 3, room, FALSE, FALSE);
+
+	clipped = nrealloc(clipped, strlen(clipped) + 4);
+	memmove(clipped + 3, clipped, strlen(clipped) + 1);
+	clipped[0] = '.'; clipped[1] = '.'; clipped[2] = '.';
+
+	return clipped;
+}
+
 #ifndef NANO_TINY
 /* Delete the lockfile.  Return TRUE on success, and FALSE otherwise. */
 bool delete_lockfile(const char *lockfilename)
@@ -225,7 +246,7 @@ char *do_lockfile(const char *filename, bool ask_the_user)
 	else if (stat(lockfilename, &fileinfo) != -1) {
 		char *lockbuf, *question, *pidstring, *postedname, *promptstr;
 		static char lockprog[11], lockuser[17];
-		int lockfd, lockpid, room, choice;
+		int lockfd, lockpid, choice;
 		ssize_t readamt;
 
 		if ((lockfd = open(lockfilename, O_RDONLY)) < 0) {
@@ -266,19 +287,8 @@ char *do_lockfile(const char *filename, bool ask_the_user)
 
 		/* TRANSLATORS: The second %s is the name of the user, the third that of the editor. */
 		question = _("File %s is being edited by %s (with %s, PID %s); open anyway?");
-		room = COLS - breadth(question) + 7 - breadth(lockuser) -
-								breadth(lockprog) - breadth(pidstring);
-		if (room < 4)
-			postedname = copy_of("_");
-		else if (room < breadth(filename)) {
-			char *fragment = display_string(filename,
-								breadth(filename) - room + 3, room, FALSE, FALSE);
-			postedname = nmalloc(strlen(fragment) + 4);
-			strcpy(postedname, "...");
-			strcat(postedname, fragment);
-			free(fragment);
-		} else
-			postedname = display_string(filename, 0, room, FALSE, FALSE);
+		postedname = crop_to_fit(filename, COLS - breadth(question) - breadth(lockuser) -
+											breadth(lockprog) - breadth(pidstring) + 7);
 
 		/* Allow extra space for username (14), program name (8), PID (8),
 		 * and terminating \0 (1), minus the %s (2) for the file name. */
@@ -488,7 +498,8 @@ void prepare_for_display(void)
 
 #ifdef ENABLE_COLOR
 	/* Precalculate the data for any multiline coloring regexes. */
-	precalc_multicolorinfo();
+	if (!openfile->filetop->multidata)
+		precalc_multicolorinfo();
 	have_palette = FALSE;
 #endif
 	refresh_needed = TRUE;
@@ -500,7 +511,13 @@ void mention_name_and_linecount(void)
 {
 	size_t count = openfile->filebot->lineno -
 						(openfile->filebot->data[0] == '\0' ? 1 : 0);
+
 #ifndef NANO_TINY
+	if (ISSET(MINIBAR)) {
+		report_size = TRUE;
+		return;
+	}
+
 	if (openfile->fmt != NIX_FILE)
 		/* TRANSLATORS: First %s is file name, second %s is file format. */
 		statusline(HUSH, P_("%s -- %zu line (%s)", "%s -- %zu lines (%s)", count),
@@ -519,7 +536,7 @@ void redecorate_after_switch(void)
 {
 	/* If only one file buffer is open, there is nothing to update. */
 	if (openfile == openfile->next) {
-		statusbar(_("No more open file buffers"));
+		statusline(AHEM, _("No more open file buffers"));
 		return;
 	}
 
@@ -722,7 +739,7 @@ void read_file(FILE *f, int fd, const char *filename, bool undoable)
 
 	/* When reading from stdin, restore the terminal and reenter curses mode. */
 	if (isendwin()) {
-		if (!isatty(STANDARD_INPUT))
+		if (!isatty(STDIN_FILENO))
 			reconnect_and_store_state();
 		terminal_init();
 		doupdate();
@@ -802,6 +819,10 @@ void read_file(FILE *f, int fd, const char *filename, bool undoable)
 	/* If we inserted less than a screenful, don't center the cursor. */
 	if (undoable && less_than_a_screenful(was_lineno, was_leftedge))
 		focusing = FALSE;
+#ifdef ENABLE_COLOR
+	else if (undoable)
+		precalc_multicolorinfo();
+#endif
 
 #ifndef NANO_TINY
 	if (undoable)
@@ -831,7 +852,7 @@ int open_file(const char *filename, bool new_one, FILE **f)
 		free(full_filename);
 
 		if (new_one) {
-			statusbar(_("New File"));
+			statusline(REMARK, _("New File"));
 			return 0;
 		} else {
 			statusline(ALERT, _("File \"%s\" not found"), filename);
@@ -966,13 +987,16 @@ bool execute_command(const char *command)
 		close(from_fd[0]);
 
 		/* Connect the write end of the output pipe to the process' output streams. */
-		dup2(from_fd[1], fileno(stdout));
-		dup2(from_fd[1], fileno(stderr));
+		if (dup2(from_fd[1], STDOUT_FILENO) < 0)
+			exit(3);
+		if (dup2(from_fd[1], STDERR_FILENO) < 0)
+			exit(4);
 
 		/* If the parent sends text, connect the read end of the
 		 * feeding pipe to the child's input stream. */
 		if (should_pipe) {
-			dup2(to_fd[0], fileno(stdin));
+			if (dup2(to_fd[0], STDIN_FILENO) < 0)
+				exit(5);
 			close(to_fd[1]);
 		}
 
@@ -980,7 +1004,7 @@ bool execute_command(const char *command)
 		execl(theshell, tail(theshell), "-c", should_pipe ? &command[1] : command, NULL);
 
 		/* If the exec call returns, there was an error. */
-		exit(1);
+		exit(6);
 	}
 
 	/* Parent: close the unused write end of the pipe. */
@@ -1978,8 +2002,13 @@ bool write_file(const char *name, FILE *thefile, bool tmp,
 		titlebar(NULL);
 	}
 
+#ifndef NANO_TINY
+	if (ISSET(MINIBAR) && fullbuffer && !tmp)
+		report_size = TRUE;
+	else
+#endif
 	if (!tmp)
-		statusline(HUSH, P_("Wrote %zu line", "Wrote %zu lines",
+		statusline(REMARK, P_("Wrote %zu line", "Wrote %zu lines",
 								lineswritten), lineswritten);
 	retval = TRUE;
 
@@ -2043,12 +2072,11 @@ bool write_marked_file(const char *name, FILE *stream, bool tmp,
  * the buffer is to be discarded. */
 int do_writeout(bool exiting, bool withprompt)
 {
-	bool result = FALSE;
-	kind_of_writing_type method = OVERWRITE;
 	char *given;
 		/* The filename we offer, or what the user typed so far. */
 	bool maychange = (openfile->filename[0] == '\0');
 		/* Whether it's okay to save the file under a different name. */
+	kind_of_writing_type method = OVERWRITE;
 #ifdef ENABLE_EXTRA
 	static bool did_credits = FALSE;
 #endif
@@ -2105,7 +2133,8 @@ int do_writeout(bool exiting, bool withprompt)
 
 		if (response < 0) {
 			statusbar(_("Cancelled"));
-			break;
+			free(given);
+			return 0;
 		}
 
 		func = func_from_key(&response);
@@ -2147,9 +2176,8 @@ int do_writeout(bool exiting, bool withprompt)
 			continue;
 		}
 #endif
-		if (func == do_help) {
+		if (func == do_help)
 			continue;
-		}
 
 #ifdef ENABLE_EXTRA
 		/* If the user pressed Ctrl-X in the edit window, and answered "Y" at
@@ -2162,8 +2190,10 @@ int do_writeout(bool exiting, bool withprompt)
 				did_credits = TRUE;
 			} else
 				/* TRANSLATORS: Concisely say the screen is too small. */
-				statusbar(_("Too tiny"));
-			break;
+				statusline(AHEM, _("Too tiny"));
+
+			free(given);
+			return 0;
 		}
 #endif
 
@@ -2176,6 +2206,7 @@ int do_writeout(bool exiting, bool withprompt)
 			full_filename = get_full_path(openfile->filename);
 			name_exists = (stat((full_answer == NULL) ?
 								answer : full_answer, &st) != -1);
+
 			if (openfile->filename[0] == '\0')
 				do_warning = name_exists;
 			else
@@ -2211,10 +2242,8 @@ int do_writeout(bool exiting, bool withprompt)
 
 				if (name_exists) {
 					char *question = _("File \"%s\" exists; OVERWRITE? ");
-					char *name = display_string(answer, 0,
-										COLS - breadth(question) + 1, FALSE, FALSE);
-					char *message = nmalloc(strlen(question) +
-												strlen(name) + 1);
+					char *name = crop_to_fit(answer, COLS - breadth(question) + 1);
+					char *message = nmalloc(strlen(question) + strlen(name) + 1);
 
 					sprintf(message, question, name);
 
@@ -2262,23 +2291,19 @@ int do_writeout(bool exiting, bool withprompt)
 #endif
 		}
 
-		/* Here's where we allow the selected text to be written to
-		 * a separate file.  If we're using restricted mode, this
-		 * function is disabled, since it allows reading from or
-		 * writing to files not specified on the command line. */
-#ifndef NANO_TINY
-		if (openfile->mark && !exiting && withprompt && !ISSET(RESTRICTED))
-			result = write_marked_file(answer, NULL, FALSE, method);
-		else
-#endif
-			result = write_file(answer, NULL, FALSE, method, TRUE);
-
+		free(given);
 		break;
 	}
 
-	free(given);
-
-	return result ? 1 : 0;
+	/* When the mark is on (and we've prompted for a name and we're
+	 * not exiting and we're not in restricted mode), then write out
+	 * the marked region; otherwise, write out the whole buffer. */
+#ifndef NANO_TINY
+	if (openfile->mark && withprompt && !exiting && !ISSET(RESTRICTED))
+		return write_marked_file(answer, NULL, FALSE, method);
+	else
+#endif
+		return write_file(answer, NULL, FALSE, method, TRUE);
 }
 
 /* Write the current buffer to disk, or discard it. */
